@@ -10,7 +10,7 @@ use crate::checks_gen::CheckCodePrefix;
 use crate::logging::LogLevel;
 use crate::printer::SerializationFormat;
 use crate::settings::configuration::Configuration;
-use crate::settings::types::{PatternPrefixPair, PythonVersion};
+use crate::settings::types::{PatternPrefixPair, PerFileIgnore, PythonVersion};
 
 #[derive(Debug, Parser)]
 #[command(author, about = "ruff: An extremely fast Python linter.")]
@@ -38,8 +38,10 @@ pub struct Cli {
     #[arg(short, long)]
     pub watch: bool,
     /// Attempt to automatically fix lint errors.
-    #[arg(short, long)]
-    pub fix: bool,
+    #[arg(long, overrides_with("no_fix"))]
+    fix: bool,
+    #[clap(long, overrides_with("fix"), hide = true)]
+    no_fix: bool,
     /// Disable cache reads.
     #[arg(short, long)]
     pub no_cache: bool,
@@ -94,6 +96,22 @@ pub struct Cli {
     pub stdin_filename: Option<String>,
 }
 
+impl Cli {
+    // See: https://github.com/clap-rs/clap/issues/3146
+    pub fn fix(&self) -> Option<bool> {
+        resolve_bool_arg(self.fix, self.no_fix)
+    }
+}
+
+fn resolve_bool_arg(yes: bool, no: bool) -> Option<bool> {
+    match (yes, no) {
+        (true, false) => Some(true),
+        (false, true) => Some(false),
+        (false, false) => None,
+        (..) => unreachable!("Clap should make this impossible"),
+    }
+}
+
 /// Map the CLI settings to a `LogLevel`.
 pub fn extract_log_level(cli: &Cli) -> LogLevel {
     if cli.silent {
@@ -165,10 +183,11 @@ pub fn warn_on(
     }
 }
 
-/// Collect a list of `PatternPrefixPair` structs as a `BTreeMap`.
+/// Convert a list of `PatternPrefixPair` structs to `PerFileIgnore`.
 pub fn collect_per_file_ignores(
     pairs: Vec<PatternPrefixPair>,
-) -> BTreeMap<String, Vec<CheckCodePrefix>> {
+    project_root: &Option<PathBuf>,
+) -> Vec<PerFileIgnore> {
     let mut per_file_ignores: BTreeMap<String, Vec<CheckCodePrefix>> = BTreeMap::new();
     for pair in pairs {
         per_file_ignores
@@ -177,4 +196,7 @@ pub fn collect_per_file_ignores(
             .push(pair.prefix);
     }
     per_file_ignores
+        .iter()
+        .map(|(pattern, prefixes)| PerFileIgnore::new(pattern, prefixes, project_root))
+        .collect()
 }
