@@ -1,25 +1,13 @@
 use std::collections::BTreeSet;
 
-use itertools::izip;
 use regex::Regex;
 use rustpython_parser::ast::{
-    Arg, Arguments, Cmpop, Constant, Excepthandler, ExcepthandlerKind, Expr, ExprKind, Stmt,
-    StmtKind,
+    Arg, Arguments, Constant, Excepthandler, ExcepthandlerKind, Expr, ExprKind, Stmt, StmtKind,
 };
 
-use crate::ast::types::{BindingKind, CheckLocator, FunctionScope, Range, Scope, ScopeKind};
+use crate::ast::types::{BindingKind, FunctionScope, Range, Scope, ScopeKind};
 use crate::checks::{Check, CheckKind};
 
-/// F634
-pub fn if_tuple(test: &Expr, location: Range) -> Option<Check> {
-    if let ExprKind::Tuple { elts, .. } = &test.node {
-        if !elts.is_empty() {
-            return Some(Check::new(CheckKind::IfTuple, location));
-        }
-    }
-    None
-}
-
 /// F631
 pub fn assert_tuple(test: &Expr, location: Range) -> Option<Check> {
     if let ExprKind::Tuple { elts, .. } = &test.node {
@@ -30,12 +18,40 @@ pub fn assert_tuple(test: &Expr, location: Range) -> Option<Check> {
     None
 }
 
+/// F634
+pub fn if_tuple(test: &Expr, location: Range) -> Option<Check> {
+    if let ExprKind::Tuple { elts, .. } = &test.node {
+        if !elts.is_empty() {
+            return Some(Check::new(CheckKind::IfTuple, location));
+        }
+    }
+    None
+}
+
+/// F821
+pub fn undefined_local(scopes: &[&Scope], name: &str) -> Option<Check> {
+    let current = &scopes.last().expect("No current scope found.");
+    if matches!(current.kind, ScopeKind::Function(_)) && !current.values.contains_key(name) {
+        for scope in scopes.iter().rev().skip(1) {
+            if matches!(scope.kind, ScopeKind::Function(_) | ScopeKind::Module) {
+                if let Some(binding) = scope.values.get(name) {
+                    if let Some((scope_id, location)) = binding.used {
+                        if scope_id == current.id {
+                            return Some(Check::new(
+                                CheckKind::UndefinedLocal(name.to_string()),
+                                location,
+                            ));
+                        }
+                    }
+                }
+            }
+        }
+    }
+    None
+}
+
 /// F841
-pub fn unused_variables(
-    scope: &Scope,
-    locator: &dyn CheckLocator,
-    dummy_variable_rgx: &Regex,
-) -> Vec<Check> {
+pub fn unused_variables(scope: &Scope, dummy_variable_rgx: &Regex) -> Vec<Check> {
     let mut checks: Vec<Check> = vec![];
 
     if matches!(
@@ -55,7 +71,7 @@ pub fn unused_variables(
         {
             checks.push(Check::new(
                 CheckKind::UnusedVariable(name.to_string()),
-                locator.locate_check(binding.range),
+                binding.range,
             ));
         }
     }
@@ -78,33 +94,6 @@ pub fn default_except_not_last(handlers: &[Excepthandler]) -> Option<Check> {
     None
 }
 
-/// F901
-pub fn raise_not_implemented(expr: &Expr) -> Option<Check> {
-    match &expr.node {
-        ExprKind::Call { func, .. } => {
-            if let ExprKind::Name { id, .. } = &func.node {
-                if id == "NotImplemented" {
-                    return Some(Check::new(
-                        CheckKind::RaiseNotImplemented,
-                        Range::from_located(expr),
-                    ));
-                }
-            }
-        }
-        ExprKind::Name { id, .. } => {
-            if id == "NotImplemented" {
-                return Some(Check::new(
-                    CheckKind::RaiseNotImplemented,
-                    Range::from_located(expr),
-                ));
-            }
-        }
-        _ => {}
-    }
-
-    None
-}
-
 /// F831
 pub fn duplicate_arguments(arguments: &Arguments) -> Vec<Check> {
     let mut checks: Vec<Check> = vec![];
@@ -158,7 +147,6 @@ pub fn repeated_keys(
     keys: &[Expr],
     check_repeated_literals: bool,
     check_repeated_variables: bool,
-    locator: &dyn CheckLocator,
 ) -> Vec<Check> {
     let mut checks: Vec<Check> = vec![];
 
@@ -173,7 +161,7 @@ pub fn repeated_keys(
                     if check_repeated_literals && v1 == v2 {
                         checks.push(Check::new(
                             CheckKind::MultiValueRepeatedKeyLiteral,
-                            locator.locate_check(Range::from_located(k2)),
+                            Range::from_located(k2),
                         ))
                     }
                 }
@@ -181,7 +169,7 @@ pub fn repeated_keys(
                     if check_repeated_variables && v1 == v2 {
                         checks.push(Check::new(
                             CheckKind::MultiValueRepeatedKeyVariable((*v2).to_string()),
-                            locator.locate_check(Range::from_located(k2)),
+                            Range::from_located(k2),
                         ))
                     }
                 }
@@ -193,45 +181,6 @@ pub fn repeated_keys(
     checks
 }
 
-fn is_constant(expr: &Expr) -> bool {
-    match &expr.node {
-        ExprKind::Constant { .. } => true,
-        ExprKind::Tuple { elts, .. } => elts.iter().all(is_constant),
-        _ => false,
-    }
-}
-
-fn is_singleton(expr: &Expr) -> bool {
-    matches!(
-        expr.node,
-        ExprKind::Constant {
-            value: Constant::None | Constant::Bool(_) | Constant::Ellipsis,
-            ..
-        }
-    )
-}
-
-fn is_constant_non_singleton(expr: &Expr) -> bool {
-    is_constant(expr) && !is_singleton(expr)
-}
-
-/// F632
-pub fn is_literal(left: &Expr, ops: &[Cmpop], comparators: &[Expr], location: Range) -> Vec<Check> {
-    let mut checks: Vec<Check> = vec![];
-
-    let mut left = left;
-    for (op, right) in izip!(ops, comparators) {
-        if matches!(op, Cmpop::Is | Cmpop::IsNot)
-            && (is_constant_non_singleton(left) || is_constant_non_singleton(right))
-        {
-            checks.push(Check::new(CheckKind::IsLiteral, location));
-        }
-        left = right;
-    }
-
-    checks
-}
-
 /// F621, F622
 pub fn starred_expressions(
     elts: &[Expr],
@@ -263,12 +212,7 @@ pub fn starred_expressions(
 }
 
 /// F701
-pub fn break_outside_loop(
-    stmt: &Stmt,
-    parents: &[&Stmt],
-    parent_stack: &[usize],
-    locator: &dyn CheckLocator,
-) -> Option<Check> {
+pub fn break_outside_loop(stmt: &Stmt, parents: &[&Stmt], parent_stack: &[usize]) -> Option<Check> {
     let mut allowed: bool = false;
     let mut parent = stmt;
     for index in parent_stack.iter().rev() {
@@ -296,7 +240,7 @@ pub fn break_outside_loop(
     if !allowed {
         Some(Check::new(
             CheckKind::BreakOutsideLoop,
-            locator.locate_check(Range::from_located(stmt)),
+            Range::from_located(stmt),
         ))
     } else {
         None
@@ -308,7 +252,6 @@ pub fn continue_outside_loop(
     stmt: &Stmt,
     parents: &[&Stmt],
     parent_stack: &[usize],
-    locator: &dyn CheckLocator,
 ) -> Option<Check> {
     let mut allowed: bool = false;
     let mut parent = stmt;
@@ -337,7 +280,7 @@ pub fn continue_outside_loop(
     if !allowed {
         Some(Check::new(
             CheckKind::ContinueOutsideLoop,
-            locator.locate_check(Range::from_located(stmt)),
+            Range::from_located(stmt),
         ))
     } else {
         None
