@@ -17,9 +17,7 @@ pub fn super_call_with_parameters(checker: &mut Checker, expr: &Expr, func: &Exp
             .collect();
         if let Some(mut check) = checks::super_args(scope, &parents, expr, func, args) {
             if checker.patch() {
-                if let Some(fix) =
-                    pyupgrade::fixes::remove_super_arguments(checker.get_locator(), expr)
-                {
+                if let Some(fix) = pyupgrade::fixes::remove_super_arguments(checker.locator, expr) {
                     check.amend(fix);
                 }
             }
