@@ -1,15 +1,13 @@
 use rustpython_ast::Expr;
 
-use crate::ast::types::{CheckLocator, Range};
+use crate::ast::types::Range;
 use crate::autofix::Fix;
 use crate::check_ast::Checker;
 use crate::checks::CheckKind;
 use crate::pyupgrade::checks;
 
 pub fn type_of_primitive(checker: &mut Checker, expr: &Expr, func: &Expr, args: &[Expr]) {
-    if let Some(mut check) =
-        checks::type_of_primitive(func, args, checker.locate_check(Range::from_located(expr)))
-    {
+    if let Some(mut check) = checks::type_of_primitive(func, args, Range::from_located(expr)) {
         if checker.patch() {
             if let CheckKind::TypeOfPrimitive(primitive) = &check.kind {
                 check.amend(Fix::replacement(
