@@ -1,6 +1,5 @@
 use rustpython_ast::{Constant, Expr, ExprKind, Operator};
 
-use crate::ast::helpers::match_name_or_attr;
 use crate::ast::types::Range;
 use crate::autofix::Fix;
 use crate::check_ast::Checker;
@@ -43,7 +42,7 @@ fn union(elts: &[Expr]) -> Expr {
 }
 
 pub fn use_pep604_annotation(checker: &mut Checker, expr: &Expr, value: &Expr, slice: &Expr) {
-    if match_name_or_attr(value, "Optional") {
+    if checker.match_typing_module(value, "Optional") {
         let mut check = Check::new(CheckKind::UsePEP604Annotation, Range::from_located(expr));
         if checker.patch() {
             let mut generator = SourceGenerator::new();
@@ -58,7 +57,7 @@ pub fn use_pep604_annotation(checker: &mut Checker, expr: &Expr, value: &Expr, s
             }
         }
         checker.add_check(check);
-    } else if match_name_or_attr(value, "Union") {
+    } else if checker.match_typing_module(value, "Union") {
         let mut check = Check::new(CheckKind::UsePEP604Annotation, Range::from_located(expr));
         if checker.patch() {
             match &slice.node {
