@@ -2,7 +2,6 @@
 //! command-line options. Structure mirrors the user-facing representation of
 //! the various parameters.
 
-use std::collections::BTreeMap;
 use std::path::PathBuf;
 
 use anyhow::{anyhow, Result};
@@ -11,8 +10,8 @@ use regex::Regex;
 
 use crate::checks_gen::CheckCodePrefix;
 use crate::settings::pyproject::load_options;
-use crate::settings::types::{FilePattern, PythonVersion};
-use crate::{flake8_quotes, pep8_naming};
+use crate::settings::types::{FilePattern, PerFileIgnore, PythonVersion};
+use crate::{flake8_annotations, flake8_quotes, pep8_naming};
 
 #[derive(Debug)]
 pub struct Configuration {
@@ -21,12 +20,14 @@ pub struct Configuration {
     pub extend_exclude: Vec<FilePattern>,
     pub extend_ignore: Vec<CheckCodePrefix>,
     pub extend_select: Vec<CheckCodePrefix>,
+    pub fix: bool,
     pub ignore: Vec<CheckCodePrefix>,
     pub line_length: usize,
-    pub per_file_ignores: BTreeMap<String, Vec<CheckCodePrefix>>,
+    pub per_file_ignores: Vec<PerFileIgnore>,
     pub select: Vec<CheckCodePrefix>,
     pub target_version: PythonVersion,
     // Plugins
+    pub flake8_annotations: flake8_annotations::settings::Settings,
     pub flake8_quotes: flake8_quotes::settings::Settings,
     pub pep8_naming: pep8_naming::settings::Settings,
 }
@@ -91,10 +92,25 @@ impl Configuration {
                 .select
                 .unwrap_or_else(|| vec![CheckCodePrefix::E, CheckCodePrefix::F]),
             extend_select: options.extend_select.unwrap_or_default(),
+            fix: options.fix.unwrap_or_default(),
             ignore: options.ignore.unwrap_or_default(),
             line_length: options.line_length.unwrap_or(88),
-            per_file_ignores: options.per_file_ignores.unwrap_or_default(),
+            per_file_ignores: options
+                .per_file_ignores
+                .map(|per_file_ignores| {
+                    per_file_ignores
+                        .iter()
+                        .map(|(pattern, prefixes)| {
+                            PerFileIgnore::new(pattern, prefixes, project_root)
+                        })
+                        .collect()
+                })
+                .unwrap_or_default(),
             // Plugins
+            flake8_annotations: options
+                .flake8_annotations
+                .map(flake8_annotations::settings::Settings::from_options)
+                .unwrap_or_default(),
             flake8_quotes: options
                 .flake8_quotes
                 .map(flake8_quotes::settings::Settings::from_options)
