@@ -2,7 +2,7 @@
 //! command-line options. Structure is optimized for internal usage, as opposed
 //! to external visibility or parsing.
 
-use std::collections::{BTreeMap, BTreeSet};
+use std::collections::BTreeSet;
 use std::hash::{Hash, Hasher};
 
 use regex::Regex;
@@ -11,7 +11,7 @@ use crate::checks::CheckCode;
 use crate::checks_gen::{CheckCodePrefix, PrefixSpecificity};
 use crate::settings::configuration::Configuration;
 use crate::settings::types::{FilePattern, PerFileIgnore, PythonVersion};
-use crate::{flake8_quotes, pep8_naming};
+use crate::{flake8_annotations, flake8_quotes, pep8_naming};
 
 pub mod configuration;
 pub mod options;
@@ -29,6 +29,7 @@ pub struct Settings {
     pub per_file_ignores: Vec<PerFileIgnore>,
     pub target_version: PythonVersion,
     // Plugins
+    pub flake8_annotations: flake8_annotations::settings::Settings,
     pub flake8_quotes: flake8_quotes::settings::Settings,
     pub pep8_naming: pep8_naming::settings::Settings,
 }
@@ -45,10 +46,11 @@ impl Settings {
             ),
             exclude: config.exclude,
             extend_exclude: config.extend_exclude,
+            flake8_annotations: config.flake8_annotations,
             flake8_quotes: config.flake8_quotes,
             line_length: config.line_length,
             pep8_naming: config.pep8_naming,
-            per_file_ignores: resolve_per_file_ignores(&config.per_file_ignores),
+            per_file_ignores: config.per_file_ignores,
             target_version: config.target_version,
         }
     }
@@ -62,6 +64,7 @@ impl Settings {
             line_length: 88,
             per_file_ignores: Default::default(),
             target_version: PythonVersion::Py310,
+            flake8_annotations: Default::default(),
             flake8_quotes: Default::default(),
             pep8_naming: Default::default(),
         }
@@ -76,6 +79,7 @@ impl Settings {
             line_length: 88,
             per_file_ignores: Default::default(),
             target_version: PythonVersion::Py310,
+            flake8_annotations: Default::default(),
             flake8_quotes: Default::default(),
             pep8_naming: Default::default(),
         }
@@ -95,6 +99,7 @@ impl Hash for Settings {
         }
         self.target_version.hash(state);
         // Add plugin properties in alphabetical order.
+        self.flake8_annotations.hash(state);
         self.flake8_quotes.hash(state);
         self.pep8_naming.hash(state);
     }
@@ -143,15 +148,6 @@ fn resolve_codes(
     codes
 }
 
-fn resolve_per_file_ignores(
-    per_file_ignores: &BTreeMap<String, Vec<CheckCodePrefix>>,
-) -> Vec<PerFileIgnore> {
-    per_file_ignores
-        .iter()
-        .map(|(pattern, prefixes)| PerFileIgnore::new(pattern, prefixes, &None))
-        .collect()
-}
-
 #[cfg(test)]
 mod tests {
     use std::collections::BTreeSet;
