@@ -134,6 +134,7 @@ mod tests {
             Some(Tools {
                 ruff: Some(Options {
                     line_length: None,
+                    fix: None,
                     exclude: None,
                     extend_exclude: None,
                     select: None,
@@ -143,6 +144,7 @@ mod tests {
                     per_file_ignores: None,
                     dummy_variable_rgx: None,
                     target_version: None,
+                    flake8_annotations: None,
                     flake8_quotes: None,
                     pep8_naming: None,
                 })
@@ -161,6 +163,7 @@ line-length = 79
             Some(Tools {
                 ruff: Some(Options {
                     line_length: Some(79),
+                    fix: None,
                     exclude: None,
                     extend_exclude: None,
                     select: None,
@@ -170,6 +173,7 @@ line-length = 79
                     per_file_ignores: None,
                     dummy_variable_rgx: None,
                     target_version: None,
+                    flake8_annotations: None,
                     flake8_quotes: None,
                     pep8_naming: None,
                 })
@@ -188,6 +192,7 @@ exclude = ["foo.py"]
             Some(Tools {
                 ruff: Some(Options {
                     line_length: None,
+                    fix: None,
                     exclude: Some(vec!["foo.py".to_string()]),
                     extend_exclude: None,
                     select: None,
@@ -197,6 +202,7 @@ exclude = ["foo.py"]
                     per_file_ignores: None,
                     dummy_variable_rgx: None,
                     target_version: None,
+                    flake8_annotations: None,
                     flake8_quotes: None,
                     pep8_naming: None,
                 })
@@ -215,6 +221,7 @@ select = ["E501"]
             Some(Tools {
                 ruff: Some(Options {
                     line_length: None,
+                    fix: None,
                     exclude: None,
                     extend_exclude: None,
                     select: Some(vec![CheckCodePrefix::E501]),
@@ -224,6 +231,7 @@ select = ["E501"]
                     per_file_ignores: None,
                     dummy_variable_rgx: None,
                     target_version: None,
+                    flake8_annotations: None,
                     flake8_quotes: None,
                     pep8_naming: None,
                 })
@@ -243,6 +251,7 @@ ignore = ["E501"]
             Some(Tools {
                 ruff: Some(Options {
                     line_length: None,
+                    fix: None,
                     exclude: None,
                     extend_exclude: None,
                     select: None,
@@ -252,6 +261,7 @@ ignore = ["E501"]
                     per_file_ignores: None,
                     dummy_variable_rgx: None,
                     target_version: None,
+                    flake8_annotations: None,
                     flake8_quotes: None,
                     pep8_naming: None,
                 })
@@ -310,6 +320,7 @@ other-attribute = 1
             config,
             Options {
                 line_length: Some(88),
+                fix: None,
                 exclude: None,
                 extend_exclude: Some(vec![
                     "excluded.py".to_string(),
@@ -326,6 +337,7 @@ other-attribute = 1
                 ),])),
                 dummy_variable_rgx: None,
                 target_version: None,
+                flake8_annotations: None,
                 flake8_quotes: Some(flake8_quotes::settings::Options {
                     inline_quotes: Some(Quote::Single),
                     multiline_quotes: Some(Quote::Double),
