@@ -1,13 +1,13 @@
 //! Structs to render user-facing settings.
 
-use std::collections::BTreeMap;
 use std::path::PathBuf;
 
 use regex::Regex;
 
+use crate::checks::CheckCode;
 use crate::checks_gen::CheckCodePrefix;
 use crate::settings::types::{FilePattern, PythonVersion};
-use crate::{flake8_quotes, pep8_naming, Configuration};
+use crate::{flake8_annotations, flake8_quotes, pep8_naming, Configuration};
 
 /// Struct to render user-facing exclusion patterns.
 #[derive(Debug)]
@@ -40,12 +40,14 @@ pub struct UserConfiguration {
     pub extend_exclude: Vec<Exclusion>,
     pub extend_ignore: Vec<CheckCodePrefix>,
     pub extend_select: Vec<CheckCodePrefix>,
+    pub fix: bool,
     pub ignore: Vec<CheckCodePrefix>,
     pub line_length: usize,
-    pub per_file_ignores: BTreeMap<String, Vec<CheckCodePrefix>>,
+    pub per_file_ignores: Vec<(Exclusion, Vec<CheckCode>)>,
     pub select: Vec<CheckCodePrefix>,
     pub target_version: PythonVersion,
     // Plugins
+    pub flake8_annotations: flake8_annotations::settings::Settings,
     pub flake8_quotes: flake8_quotes::settings::Settings,
     pub pep8_naming: pep8_naming::settings::Settings,
     // Non-settings exposed to the user
@@ -73,11 +75,22 @@ impl UserConfiguration {
                 .collect(),
             extend_ignore: configuration.extend_ignore,
             extend_select: configuration.extend_select,
+            fix: configuration.fix,
             ignore: configuration.ignore,
             line_length: configuration.line_length,
-            per_file_ignores: configuration.per_file_ignores,
+            per_file_ignores: configuration
+                .per_file_ignores
+                .into_iter()
+                .map(|per_file_ignore| {
+                    (
+                        Exclusion::from_file_pattern(per_file_ignore.pattern),
+                        Vec::from_iter(per_file_ignore.codes),
+                    )
+                })
+                .collect(),
             select: configuration.select,
             target_version: configuration.target_version,
+            flake8_annotations: configuration.flake8_annotations,
             flake8_quotes: configuration.flake8_quotes,
             pep8_naming: configuration.pep8_naming,
             project_root,
